#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <glib.h>
#include <sqlite.h>
#include <gpe/errorbox.h>
#include "db.h"

#define DB_NAME_PHONE "/.gpe/phone"
#define DB_NAME_CONTACTS "/.gpe/contacts"

static const char *schema_call = "create table call (id INTEGER PRIMARY KEY, list INTEGER, seen INTEGER, number TEXT, time TIMESTAMP, duration INTEGER);";
static const char *schema_sms = "create table sms (id INTEGER PRIMARY KEY, folder TEXT, seen INTEGER, pdu_data TEXT, time TIMESTAMP);";

static sqlite *db = NULL;
static sqlite *dbc = NULL;

int db_open(void)
{
	/* open persistent connection */
	char *errmsg;
	char *buf;

	buf = g_strdup_printf("%s/%s", g_get_home_dir(), DB_NAME_PHONE);
	if (db)
		sqlite_close(db);
	db = sqlite_open(buf, 0, &errmsg);
	g_free(buf);

	buf = g_strdup_printf("%s/%s", g_get_home_dir(), DB_NAME_CONTACTS);
	if (dbc)
		sqlite_close(dbc);
	dbc = sqlite_open(buf, 0, &errmsg);
	g_free(buf);

	if (db == NULL) {
		gpe_error_box(errmsg);
		free(errmsg);
		return -1;
	}

	sqlite_exec(db, schema_call, NULL, NULL, NULL);
	sqlite_exec(db, schema_sms, NULL, NULL, NULL);

	return 0;
}


int
db_call_new(char *number, int list)
{
	int r;
	char *err;

	r = sqlite_exec_printf (db, "insert into call (number,list,time,duration) values ('%s',%d,strftime('%%s','now'),0)", NULL, NULL, &err, number, list);
	if (r) {
		gpe_error_box (err);
		free (err);
		return -1;
	}
	return sqlite_last_insert_rowid (db);
}

int
db_call_update_number(int callid, char *number)
{
	char *err;
	int r;

	if (callid == -1)
		return -1;
	
	r = sqlite_exec_printf (db, "update call set number='%s' where id=%d", NULL, NULL, &err, number, callid);
	printf ("update call set number='%s' where id=%d\n", number, callid);
	if (r) {
		gpe_error_box (err);
		free (err);
		return -1;
	}
	return 0;
}

int
db_call_update_list_time(int callid, int list)
{
	char *err;
	int r;

	if (callid == -1)
		return -1;
	
	r = sqlite_exec_printf (db, "update call set list=%d,time=strftime('%%s','now') where id=%d", NULL, NULL, &err, list, callid);
	printf ("update call set list=%d,time=strftime('%%s','now') where id=%d\n", list, callid);
	if (r) {
		gpe_error_box (err);
		free (err);
		return -1;
	}
	return 0;
}

int
db_call_update_list_duration(int callid, int list)
{
	char *err;
	int r;

	if (callid == -1)
		return -1;
	
	r = sqlite_exec_printf (db, "update call set list=%d,duration=strftime('%%s','now')-time where id=%d", NULL, NULL, &err, list, callid);
	if (r) {
		gpe_error_box (err);
		free (err);
		return -1;
	}
	return 0;
}

int
db_call_update_duration(int callid)
{
	char *err;
	int r;

	printf("db_call_update_duration(%d)\n", callid);
	if (callid == -1)
		return -1;

	r = sqlite_exec_printf (db, "update call set duration=strftime('%%s','now')-time where id=%d", NULL, NULL, &err, callid);
	if (r) {
		gpe_error_box (err);
		free (err);
		return -1;
	}
	return 0;
}

static int
db_call_query_add(void *arg, int argc, char **argv, char **names)
{
	struct call_entry *e;
	GSList **list = (GSList **) arg;
	e=g_malloc0(sizeof(*e));
	if (argv[0]) e->time=atoi(argv[0]);
	if (argv[1]) e->number=g_strdup(argv[1]);
	if (argv[2]) e->duration=atoi(argv[2]);
	if (argv[3]) e->seen=atoi(argv[3]);
	*list=g_slist_prepend(*list, e);

	return 0;
}

GSList *
db_call_query(int list)
{
	char *err;
	int r;
	GSList *ret=NULL;

	printf("db_call_query\n");
	r = sqlite_exec_printf (db, "select time,number,duration,seen from call where (list & %d)<>0 order by time", db_call_query_add, &ret, &err, list);
	printf("r=%d err=%p\n", r, err);
	if (r) {
		gpe_error_box (err);
		free (err);
		return NULL;
	}

	return ret;
}

void
db_call_entry_free(struct call_entry *e)
{
	g_free(e->number);
	g_free(e);
}


int
db_sms_new(char *folder, char *pdu_data)
{
	int r;
	char *err;

	r = sqlite_exec_printf (db, "insert into sms (folder,pdu_data,time) values ('%s','%s',strftime('%%s','now'))", NULL, NULL, &err, folder, pdu_data);
	if (r) {
		gpe_error_box (err);
		free (err);
		return -1;
	}
	return sqlite_last_insert_rowid (db);
}

static int
db_sms_query_add(void *arg, int argc, char **argv, char **names)
{
	struct sms_entry *e;
	GSList **list = (GSList **) arg;
	e=g_malloc0(sizeof(*e));
	if (argv[0]) e->time=atoi(argv[0]);
	if (argv[1]) e->pdu_data=g_strdup(argv[1]);
	if (argv[2]) e->seen=atoi(argv[2]);
	*list=g_slist_prepend(*list, e);
	return 0;
}

GSList *
db_sms_query(char *folder)
{
	char *err;
	int r;
	GSList *ret=NULL;

	printf("db_call_query\n");
	r = sqlite_exec_printf (db, "select time,pdu_data,seen from sms where folder='%s' order by time", db_sms_query_add, &ret, &err, folder);
	printf("r=%d err=%p\n", r, err);
	if (r) {
		gpe_error_box (err);
		free (err);
		return NULL;
	}

	return ret;
}

void
db_sms_entry_free(struct sms_entry *e)
{
	g_free(e->pdu_data);
	g_free(e);
}

static int
db_get_id(void *arg, int argc, char **argv, char **names)
{
	int *id=arg;

	if (*id == -1 && argc > 0)
		*id=atoi(argv[0]);
	return 0;
}

static int
db_get_name(void *arg, int argc, char **argv, char **names)
{
	char **name=arg;

	if (*name)
		return 0;

	printf("name %p '%s' %p '%s'\n", argv[0], argv[0], argv[1], argv[1]);
	if (argv[1]) {
		if (argv[0]) 
			*name=g_strdup_printf("%s (%s)",argv[0], argv[1]);
		else 
			*name=g_strdup_printf("(%s)", argv[1]);
	} else {
		if (argv[0])
			*name=g_strdup(argv[0]);
	}
	return 0;
}

char *
db_lookup_number(const char *number)
{
	int id=-1;
	char *tags="tag='WORK.MOBILE' or tag='WORK.TELEPHONE' or tag='HOME.MOBILE' or tag='HOME.TELEPHONE'";
	char *ret=NULL;

	if (! dbc)
		return NULL;
	sqlite_exec_printf (dbc, "select urn from contacts where (%s) and value='%s'", db_get_id, &id, NULL, tags, number);
	if (id == -1)
		sqlite_exec_printf (dbc, "select urn from contacts where (%s) and value<>'' and '%s' like '%%'||value", db_get_id, &id, NULL, tags, number);
	if (id == -1)
		sqlite_exec_printf (dbc, "select urn from contacts where (%s) and value<>'' and '%s' like '%%'||substr(value,2,255)", db_get_id, &id, NULL, tags, number);
	if (id == -1)
		return NULL;
	sqlite_exec_printf (dbc, "select name,company from contacts_urn where urn=%d", db_get_name, &ret, NULL, id);
	return ret;
	
}

void db_close(void)
{
	if (db)
		sqlite_close(db);
	if (dbc)
		sqlite_close(dbc);
}
